# src/restackio/integrations/gemini/__init__.py
__path__ = __import__('pkgutil').extend_path(__path__, __name__)

from .functions.generate_content import gemini_generate_content, GeminiGenerateContentInput
from .functions.chat.start_chat import gemini_start_chat, GeminiStartChatInput
from .functions.chat.send_message import gemini_send_message, GeminiSendMessageInput
from .functions.chat.get_history import gemini_get_history, GeminiGetHistoryInput
from .service import GeminiServiceOptions, GeminiServiceInput, gemini_service

__all__ = [
    'gemini_generate_content',
    'GeminiGenerateContentInput',
    'GeminiServiceOptions',
    'GeminiServiceInput',
    'gemini_service',
    'gemini_start_chat',
    'GeminiStartChatInput',
    'gemini_send_message',
    'GeminiSendMessageInput',
    'gemini_get_history',
    'GeminiGetHistoryInput'
]



from pydantic import BaseModel
import google.generativeai as genai

class GeminiSendMessageInput(BaseModel):
    user_content: str
    chat: genai.ChatSession
    model_config = {
        "arbitrary_types_allowed": True
    }

def gemini_send_message(input: GeminiSendMessageInput):
    return input.chat.send_message(input.user_content)




from pydantic import BaseModel
import google.generativeai as genai

class GeminiGetHistoryInput(BaseModel):
    chat: genai.ChatSession
    model_config = {
        "arbitrary_types_allowed": True
    }    

def gemini_get_history(input: GeminiGetHistoryInput):
    return input.chat.history


from restack_ai import Restack
from pydantic import BaseModel
from .functions.generate_content import gemini_generate_content
from .task_queue import gemini_task_queue

class GeminiServiceOptions(BaseModel):
    rate_limit: int

class RestackWrapper(BaseModel):
    client: Restack
    
    class Config:
        arbitrary_types_allowed = True


class GeminiServiceInput(BaseModel):
    client: RestackWrapper
    options: GeminiServiceOptions


async def gemini_service(input: GeminiServiceInput):
    return await input.client.start_service(
        functions=[
            gemini_generate_content
        ],
        task_queue=gemini_task_queue,
        options=input.options
    )

if __name__ == "__main__":
    gemini_service(
        client=Restack(),
        options=GeminiServiceOptions(rate_limit=100000)
    )



from pydantic import BaseModel
import google.generativeai as genai

class GeminiStartChatInput(BaseModel):
    model: str
    api_key: str
    generation_config: dict | None = None
    history: list[dict] | None = None

def gemini_start_chat(input: GeminiStartChatInput):
    if not input.api_key:
        raise ValueError("api_key is required")
    
    genai.configure(api_key=input.api_key)

    model = genai.GenerativeModel(
        model_name=input.model,
        generation_config=genai.GenerationConfig(**input.generation_config)
    )

    chat = model.start_chat(
        history=input.history
    )

    return chat



from pydantic import BaseModel
import google.generativeai as genai

class GeminiGenerateContentInput(BaseModel):
    user_content: str
    model: str | None = None
    api_key: str = ""
    generation_config: dict | None = None

def gemini_generate_content(input) :
    if not input.api_key:
        raise ValueError("api_key is required")
    
    genai.configure(api_key=input.api_key)

    model = genai.GenerativeModel(model_name=input.model)

    response = model.generate_content(input.user_content, generation_config=genai.GenerationConfig(**input.generation_config))

    return response.text

